#!/usr/bin/env node

import { Command } from 'commander';
import inquirer from 'inquirer';
import chalk from 'chalk';
import ora from 'ora';
import { 
  setApiKey, 
  getApiKey, 
  deleteApiKey,
  getPreferredProvider,
  getPreferredModel 
} from '../lib/config.js';
import { getUncommittedChanges, commitChanges, getDiff } from '../lib/git.js';
import { generateCommitMessage, testApiKey, setup } from '../lib/ai.js';
import { checkVersion } from '../lib/version.js';

const CURRENT_VERSION = '1.0.3';
const program = new Command();
const spinner = ora();

// Enable debug mode with environment variable
const DEBUG = process.env.DEBUG === 'true';

function handleError(error, context = '') {
  spinner.stop();
  if (DEBUG) {
    console.error(chalk.red('Debug Error:'), error);
    console.error(chalk.gray('Stack:'), error.stack);
    console.error(chalk.gray('Context:'), context);
  } else {
    console.error(chalk.red('Error:'), error.message);
    console.log(chalk.gray('Run with DEBUG=true for more information'));
  }
  process.exit(1);
}

program
  .version(CURRENT_VERSION)
  .description('A CLI tool for generating commit messages using AI')
  .option('-d, --debug', 'Enable debug mode');

// Command to setup AI provider
program
  .command('setup')
  .description('Interactive setup wizard for configuring AI providers')
  .action(async () => {
    try {
      await setup();
      await checkVersion(CURRENT_VERSION);
    } catch (error) {
      handleError(error, 'Setup Command');
    }
  });

// Command to configure AI providers
program
  .command('ai')
  .description('Configure AI provider settings')
  .option('--openai <key>', 'Set OpenAI API key')
  .option('--remove <provider>', 'Remove API key for specified provider')
  .option('--test', 'Test the configured API key')
  .action(async (options) => {
    try {
      if (options.openai) {
        spinner.start('Saving API key');
        setApiKey('openai', options.openai);
        spinner.succeed('OpenAI API key has been saved');

        // Automatically test the key after saving
        spinner.start('Testing API key');
        const isValid = await testApiKey('openai', options.openai);
        if (isValid) {
          spinner.succeed('API key is valid and working!');
        } else {
          spinner.fail('API key is invalid. Please check your key and try again');
          // Remove the invalid key
          deleteApiKey('openai');
          process.exit(1);
        }
      }
      
      if (options.remove) {
        deleteApiKey(options.remove);
        console.log(chalk.yellow(`API key for ${options.remove} has been removed.`));
      }

      if (options.test) {
        const provider = getPreferredProvider();
        const apiKey = getApiKey(provider);
        
        if (!provider || !apiKey) {
          console.error(chalk.red('Error: No AI provider configured.'));
          console.log(chalk.yellow('Please run: commit-ai setup'));
          process.exit(1);
        }

        spinner.start('Testing API key');
        const isValid = await testApiKey(provider, apiKey);
        if (isValid) {
          spinner.succeed('API key is valid and working!');
        } else {
          spinner.fail('API key is invalid or not working');
          process.exit(1);
        }
      }
      
      await checkVersion(CURRENT_VERSION);
    } catch (error) {
      handleError(error, 'AI Configuration');
    }
  });

// Command to show diff
program
  .command('diff')
  .description('Show current git changes')
  .action(async () => {
    try {
      const changes = await getUncommittedChanges();
      if (!changes.isRepo) {
        console.error(chalk.red('Error: Not a git repository'));
        process.exit(1);
      }

      const diff = await getDiff();
      if (!diff) {
        console.log(chalk.yellow('No changes to display.'));
        return;
      }

      console.log(chalk.blue('\nCurrent changes:'));
      console.log(diff);
    } catch (error) {
      handleError(error, 'Diff Command');
    }
  });

// Command to generate commit message
program
  .command('generate')
  .description('Generate a commit message')
  .option('-y, --yes', 'Skip confirmation and commit directly')
  .option('--show-diff', 'Show diff before generating commit message')
  .action(async (options) => {
    try {
      // Check if any provider is configured
      const provider = getPreferredProvider();
      const apiKey = getApiKey(provider);
      
      if (!provider || !apiKey) {
        console.error(chalk.red('Error: No AI provider configured.'));
        console.log(chalk.yellow('Please run: commit-ai setup'));
        process.exit(1);
      }

      // Get git changes
      spinner.start('Checking git repository');
      const changes = await getUncommittedChanges();
      
      if (!changes.isRepo) {
        spinner.fail('Not a git repository');
        process.exit(1);
      }

      if (![...changes.staged, ...changes.modified, ...changes.created, ...changes.deleted].length) {
        spinner.fail('No changes detected to commit.');
        process.exit(0);
      }
      spinner.succeed('Found changes to commit');

      if (options.showDiff) {
        const diff = await getDiff();
        console.log(chalk.blue('\nCurrent changes:'));
        console.log(diff);
        
        const { continue: shouldContinue } = await inquirer.prompt([
          {
            type: 'confirm',
            name: 'continue',
            message: 'Would you like to proceed with generating a commit message?',
            default: true
          }
        ]);

        if (!shouldContinue) {
          process.exit(0);
        }
      }

      spinner.start('Analyzing changes and generating commit message');
      const commitMessage = await generateCommitMessage(changes);
      spinner.succeed('Generated commit message');

      if (!options.yes) {
        const { confirm, editMessage } = await inquirer.prompt([
          {
            type: 'confirm',
            name: 'confirm',
            message: `Generated commit message:\n${chalk.green(commitMessage)}\n\nWould you like to use this message?`,
            default: true
          },
          {
            type: 'input',
            name: 'editMessage',
            message: 'Enter your modified commit message:',
            default: commitMessage,
            when: (answers) => !answers.confirm
          }
        ]);

        spinner.start('Committing changes');
        const finalMessage = confirm ? commitMessage : editMessage;
        await commitChanges(finalMessage);
        spinner.succeed('Changes committed successfully!');
      } else {
        spinner.start('Committing changes');
        await commitChanges(commitMessage);
        spinner.succeed('Changes committed successfully!');
      }

      await checkVersion(CURRENT_VERSION);
    } catch (error) {
      handleError(error, 'Generate Command');
    }
  });

program.parse(process.argv); 