import OpenAI from 'openai';
import { Anthropic } from '@anthropic-ai/sdk';
import { GoogleGenerativeAI } from '@google/generative-ai';
import { 
  getApiKey, 
  setApiKey, 
  getPreferredProvider, 
  setPreferredProvider,
  getPreferredModel,
  setPreferredModel 
} from './config.js';
import inquirer from 'inquirer';

const PROVIDERS = {
  openai: {
    name: 'OpenAI',
    defaultModel: 'gpt-4o-mini',
    createClient: (apiKey) => new OpenAI({ apiKey }),
    models: [
      { id: 'gpt-4o-mini', name: 'GPT-4o-mini (Recommended & Cheaper)' },
      { id: 'gpt-4o', name: 'GPT-4o (Most capable)' },
    ],
    async generateMessage(client, model, system, prompt) {
      const response = await client.chat.completions.create({
        model,
        messages: [
          { role: 'system', content: system },
          { role: 'user', content: prompt }
        ],
        temperature: 0.7,
        max_tokens: 100
      });
      return response.choices[0].message.content;
    }
  },
  anthropic: {
    name: 'Anthropic',
    defaultModel: 'claude-3.5-sonnet-latest',
    createClient: (apiKey) => new Anthropic({ apiKey }),
    models: [
      { id: 'claude-3-5-sonnet-latest', name: 'Claude 3.5 Sonnet (Most capable)' },
      { id: 'claude-3-5-haiku-latest', name: 'Claude 3.5 Haiku (Fastest)' }
    ],
    async generateMessage(client, model, system, prompt) {
      const response = await client.messages.create({
        model,
        max_tokens: 100,
        messages: [
          { role: 'system', content: system },
          { role: 'user', content: prompt }
        ]
      });
      return response.content[0].text;
    }
  },
  google: {
    name: 'Google',
    defaultModel: 'gemini-2.0-flash',
    createClient: (apiKey) => new GoogleGenerativeAI(apiKey),
    models: [
      { id: 'gemini-2.0-flash', name: 'Gemini 2.0 Flash (Recommended)' },
      { id: 'gemini-1.5-flash', name: 'Gemini 1.5 Flash' },
      { id: 'gemini-1.5-pro', name: 'Gemini 1.5 Pro' }
    ],
    async generateMessage(client, model, system, prompt) {
      const genModel = client.getGenerativeModel({ model });
      const response = await genModel.generateContent([
        system + "\n\n" + prompt
      ]);
      return response.response.text();
    }
  }
};

export async function getProvider(providerKey = null) {
  const provider = providerKey || getPreferredProvider();
  if (!provider || !PROVIDERS[provider]) {
    throw new Error('No provider configured. Please run: commit-ai setup');
  }
  return PROVIDERS[provider];
}

export async function testApiKey(provider, apiKey) {
  try {
    if (!PROVIDERS[provider]) {
      return false;
    }
    
    // Test with a simple generation
    const client = PROVIDERS[provider].createClient(apiKey);
    await PROVIDERS[provider].generateMessage(
      client,
      PROVIDERS[provider].defaultModel,
      'You are a helpful assistant.',
      'test'
    );
    return true;
  } catch (error) {
    if (DEBUG) {
      console.error('API Key test failed:', error.message);
    }
    return false;
  }
}

// Helper function for controlled delays
const sleep = (ms) => new Promise(resolve => setTimeout(resolve, ms));

// Helper function for smooth text printing
async function printSlowly(text, delay = 25) {
  for (let char of text) {
    process.stdout.write(char);
    await sleep(delay);
  }
  process.stdout.write('\n');
}

// Helper function for simulated loading
async function simulateLoading(text, duration = 800) {
  const frames = ['⠋', '⠙', '⠹', '⠸', '⠼', '⠴', '⠦', '⠧', '⠇', '⠏'];
  let i = 0;
  const interval = setInterval(() => {
    process.stdout.write(`\r${frames[i]} ${text}`);
    i = (i + 1) % frames.length;
  }, 60);
  
  await sleep(duration);
  clearInterval(interval);
  process.stdout.write(`\r✓ ${text}\n`);
}

export async function setup() {
  // Clear the screen
  console.clear();
  
  // Welcome message with typing effect
  await printSlowly('\n🤖 Welcome to the Commit AI Setup Wizard! 🎉');
  await sleep(300);
  await printSlowly('Let me help you configure your AI-powered commit message generator.\n');
  await sleep(500);

  // Quick environment check
  await simulateLoading('Checking environment...', 800);
  await simulateLoading('Loading provider information...', 600);

  // Provider selection - print instantly for better readability
  await printSlowly('\n📋 Available AI Providers:');
  await sleep(300);
  console.log('   - OpenAI: Known for GPT models, excellent at understanding context');
  console.log('   - Anthropic: Offers Claude models, great at technical analysis');
  console.log('   - Google: Provides Gemini models, strong at code understanding\n');
  await sleep(300);

  const { provider } = await inquirer.prompt([{
    type: 'list',
    name: 'provider',
    message: '🤔 Which AI provider would you like to use?',
    choices: Object.entries(PROVIDERS).map(([key, value]) => ({
      name: `${value.name} (recommended model: ${value.defaultModel})`,
      value: key
    }))
  }]);

  await printSlowly(`\n✨ Excellent choice! ${PROVIDERS[provider].name} is a great option.`);
  await sleep(300);

  // API Key configuration
  await printSlowly('\n🔑 Now, let\'s set up your API key.');
  const providerUrls = {
    openai: 'https://platform.openai.com/api-keys',
    anthropic: 'https://console.anthropic.com/account/keys',
    google: 'https://makersuite.google.com/app/apikey'
  };
  console.log(`You can get your API key from: ${providerUrls[provider]}\n`);

  let apiKey;
  let isValidKey = false;
  
  while (!isValidKey) {
    const result = await inquirer.prompt([{
      type: 'password',
      name: 'apiKey',
      message: '🔐 Please enter your API key:'
    }]);
    
    apiKey = result.apiKey;
    
    if (!apiKey) {
      console.log('❌ API key is required\n');
      continue;
    }
    
    await simulateLoading('Validating API key...', 1000);
    isValidKey = await testApiKey(provider, apiKey);
    
    if (!isValidKey) {
      console.log('❌ Invalid API key. Please check and try again.\n');
    }
  }

  await simulateLoading('Securing your API key...', 800);
  setApiKey(provider, apiKey);

  // Model selection - print instantly for better readability
  console.log('\n🤖 Available Models:');
  PROVIDERS[provider].models.forEach(m => {
    console.log(`   • ${m.name}`);
  });
  console.log('');
  await sleep(300);

  const { model } = await inquirer.prompt([{
    type: 'list',
    name: 'model',
    message: '📝 Which model would you like to use?',
    choices: PROVIDERS[provider].models.map(m => ({
      name: m.name,
      value: m.id
    })),
    default: PROVIDERS[provider].defaultModel
  }]);

  await simulateLoading('Saving your preferences...', 800);
  setPreferredProvider(provider);
  setPreferredModel(provider, model);

  // Final configuration summary
  await printSlowly('\n🎉 Setup completed successfully! Here\'s your configuration:');
  console.log(`   • Provider: ${PROVIDERS[provider].name}`);
  console.log(`   • Model: ${model}`);
  
  await printSlowly('\n💡 You can run this setup again anytime with: commit-ai setup');
  await printSlowly('🚀 Ready to generate awesome commit messages! Try: commit-ai\n');

  return {
    provider,
    model
  };
}

export async function generateCommitMessage(changes) {
  const providerKey = await getPreferredProvider();
  if (!providerKey || !PROVIDERS[providerKey]) {
    throw new Error('No provider configured. Please run: commit-ai setup');
  }

  const provider = PROVIDERS[providerKey];
  const model = getPreferredModel(providerKey) || provider.defaultModel;
  const apiKey = getApiKey(providerKey);
  
  if (!apiKey) {
    throw new Error('AI provider not configured. Please run: commit-ai setup');
  }

  const changesSummary = [
    changes.staged.length ? `Staged files: ${changes.staged.join(', ')}` : '',
    changes.modified.length ? `Modified files: ${changes.modified.join(', ')}` : '',
    changes.created.length ? `Created files: ${changes.created.join(', ')}` : '',
    changes.deleted.length ? `Deleted files: ${changes.deleted.join(', ')}` : '',
    `\nStaged changes:\n${changes.stagedDiff}`,
    `\nUnstaged changes:\n${changes.unstagedDiff}`
  ].filter(Boolean).join('\n');

  const client = provider.createClient(apiKey);
  const text = await provider.generateMessage(
    client,
    model,
    "You are a helpful assistant that generates clear and concise git commit messages. Follow these rules:\n" +
    "1. Use the conventional commits format (type(scope): description)\n" +
    "2. Keep the message under 72 characters\n" +
    "3. Use the imperative mood (e.g., 'add' not 'adds')\n" +
    "4. Be descriptive but concise\n" +
    "5. Focus on WHY and WHAT, not HOW",
    `Generate a commit message for these changes:\n\n${changesSummary}`
  );

  return text.trim();
} 